!   initialize: 1. source and receiver information
!               2. environmental information    
    
    subroutine initial( iflag, filenm, cwater )
    
    use gridmod
    
    implicit integer(kind=4) (i-n) 
    implicit real(kind=8)    (a-h, o-z)    
      
    ! control variables
    real(kind=8)      cwater(n)
    character*60      filenm
    
    ! common variables
    common / source / frq, zs, zr, c0, xko
    common / layer1 / rho1, alpha1
    common / layer2 / zb, cb, rho2, alpha2

    ! local variables
    real(kind=8), allocatable :: zsvp(:), csvp(:)    

    open(11, file=filenm)
    
!   source and receiver information 
    read(11, *) frq      ! frequency (Hz)
    read(11, *) zs       ! source depth (m)
    read(11, *) zr       ! receiver depth (m)
    read(11, *) c0       ! reference sound speed (m/s)     
    xko = 2.0*pi*frq/c0  ! reference wave number
    
!   layer 1 (water column)
        
    read(11, *) nsvp     ! vertical points for the observed water column
          
    if( allocated( zsvp ) ) deallocate( zsvp, stat=ierror )
    allocate( zsvp(nsvp), stat=ierror )
    zsvp = 0.d0
            
    if( allocated( csvp ) ) deallocate( csvp, stat=ierror )    
    allocate( csvp(nsvp), stat=ierror )    
    csvp = 0.d0
                
    do i=1, nsvp    
        read(11,*) zsvp(i), csvp(i)        
    end do               
    zsvp(nsvp) = zmax  ! zsvp(nsvp) should equal to zmax
             
    ! interpolate the sound speed profile to every vertical computed grid (horizontal homogeneous)
        
    j = 1    
    k = 0	
    do i=1, n    
        zi = dble(i) * dz        
        do while(( zi .gt. zsvp(j) ) .and. ( j .lt. nsvp ))        
            j = j + 1            
            k = j - 1            
        end do        
        frac = (zi - zsvp(k)) / (zsvp(j) - zsvp(k))        
        cwater(i) = csvp(k) + frac*(csvp(j)-csvp(k))         
    end do

!   layer 1 (density and antenuation)   
    read(11, *) rho1       ! density
    read(11, *) alpha1     ! antenuation
    
!   layer 2 (homogeneous medium)
    read(11, *) cb         ! sound speed of layer 2
    read(11, *) rho2       ! density of layer 2
    read(11, *) alpha2     ! antenuation of layer 2             
    zb = zmax              ! the bottom depth

    close(11)
    
    if (iflag .ne. 0) then        
        ! The Munk Profile for deep water problem test
        epsilon = 0.00737d0        
        do i = 1, n
            z = dble(i)*dz
            tem = 2.0 * (z-1300.0) / 1300.0
            tem1 = tem - 1.0 + dexp(-tem)
            cwater(i) = 1500.0 * (1.0 + epsilon * tem1)
        end do        
    end if    
    
    !   source field
    call sfield( xko, zs, n, dz, uinit )
    
    return
    
    end