!   implement of implicit CrankCNicolson finite difference scheme
    
    subroutine pestep(cwater)
    
    use gridmod
    
    implicit integer(kind=4) (i-n) 
    implicit real(kind=8)    (a-h, o-z)    
    
    ! control variables
    real(kind=8)      cwater(n)
    
    ! common variables
    common / source / frq, zs, zr, c0, xko
    common / layer1 / rho1, alpha1
    common / layer2 / zb, cb, rho2, alpha2
    
    ! local variables
    complex(kind=8) xx1, xx2, xn1, miu2, beta, theta
    complex(kind=8) a(n), xx1m(n), g1(0:nstep), g2(0:nstep)
    
!   layer 1

	xx1 = dcmplx(0.d0, 0.25/(dz*dz*xko))        ! (1/2) * b / (dz**2) 
	xx2 = 2.d0 * xx1                            ! b / (dz**2)
    
	do i=1, n		
		xn = c0 / cwater(i)
		xn1 = dcmplx( xn*xn, xn*xn*alpha1/27.287527 )
		a(i) = 0.5 * eye * xko * (xn1-1.0)
		xx1m(i) = 0.5 * a(i) - xx2
    end do
    
    ylrws = dr * xx1                            
    xlrws = -ylrws                              
    
    ymw(1) = 1.d0 + dr * xx1m(1)
    c(1,2) = 2.d0 - ymw(1)
    c(1,3) = xlrws 
	do i=2, n-1
		ymw(i) = 1.d0 + dr * xx1m(i)
		c(i,1) = xlrws
		c(i,2) = 2.d0 - ymw(i)
		c(i,3) = xlrws
    end do
    c(n-1,3) = dcmplx(0.d0, 0.d0)
    
!   interface (lower boundary)

    miu2 = 4.d0 * eye / (xko*dr)
	xn = c0 / cb                                	
	xn1 = dcmplx( xn*xn, xn*xn*alpha2/27.287527)
    beta = (rho1/rho2) * xko
    beta = beta * (xn1 - 1.d0 + miu2)**(0.5)
    
    cnbm = eye * beta * dz
    c(n-1,2) = c(n-1,2) + xlrws/(1.d0-cnbm)
    
    theta = dcmplx(1.d0,0.d0)
    call taylorcoef(theta, -0.5d0, nstep, g1)
    
    theta = (xn1-1.0-miu2) / (xn1-1.0+miu2)
    call taylorcoef(theta, 0.5d0, nstep, g2)
    
    g0 = dcmplx(0.d0,0.d0)
    do i=0, nstep
        do j=0, nstep
            k = i + j
            g0(k) = g0(k) + g1(i) * g2(j)
        end do
    end do
    
    do istep=1, nstep
	
        cr(1,istep) = uobs(1,istep-1) * ymw(1) + uobs(2,istep-1) * ylrws
        do i=2, n-1
            cr(i,istep) = uobs(i,istep-1) * ymw(i) + ( uobs(i-1,istep-1) + uobs(i+1,istep-1) ) * ylrws
        end do
        
        cnbm1 = dcmplx(0.0,0.0)
        do i=1, istep
            j = istep - i
            cnbm1 = cnbm1 + g0(i) * uobs(n,j)
        end do
        cnbm1 = cnbm1 * cnbm * xlrws / (1.0 - cnbm)
        
        cr(n-1,istep) = cr(n-1,istep) - cnbm1
        
        bet(1,istep) = c(1,2)
        ur(1,istep) = cr(1,istep) / bet(1,istep)

        do j=2, n-1
            gam(j,istep) = c(j-1,3) / bet(j-1,istep)
            bet(j,istep) = c(j,2) - c(j,1)*gam(j,istep)
            ur(j,istep) = ( cr(j,istep)-c(j,1)*ur(j-1,istep) ) / bet(j,istep)
        end do
        urm(:,istep) = ur(:,istep)
    
        do j=n-2, 1, -1
            urm(j,istep) = ur(j,istep) - gam(j+1,istep)*urm(j+1,istep)
        end do
        
        urm(n,istep) = urm(n-1,istep)/(1.0-cnbm) + cnbm1/xlrws
        
        uobs(:,istep) = urm(:,istep)
        
        do i=1, n
            tpm = uobs(i,istep) * dCONJG(uobs(i,istep))
            if ( tpm .le. 0.d0 ) then
                pobs(i,istep) = 0.d0
            else
                pobs(i,istep) = -10.0*dlog10(tpm) + 10.0*dlog10(istep*dr)
            end if
        end do
        
    end do
    
    return
    
    end subroutine pestep